=== modified file 'tarmac/bin/commands.py'
--- tarmac/bin/commands.py	2012-12-19 15:04:18 +0000
+++ tarmac/bin/commands.py	2012-12-19 15:54:44 +0000
@@ -161,8 +161,6 @@
             return
 
         target = Branch.create(lp_branch, self.config, create_tree=True)
-        # Lock the branch for writing
-        target.bzr_branch.lock_write()
 
         self.logger.debug('Firing tarmac_pre_merge hook')
         tarmac_hooks.fire('tarmac_pre_merge',
@@ -211,9 +209,10 @@
                     source = Branch.create(
                         proposal.source_branch, self.config, target=target)
 
-                    approved = source.bzr_branch.revision_id_to_revno(
+                    source_bzr_branch = source.get_bzr_branch()
+                    approved = source_bzr_branch.revision_id_to_revno(
                         str(proposal.reviewed_revid))
-                    tip = source.bzr_branch.revno()
+                    tip = source_bzr_branch.revno()
 
                     if tip > approved:
                         message = u'Unapproved changes made after approval'
@@ -228,9 +227,7 @@
                             'source': proposal.source_branch.web_link,
                             'revision': proposal.reviewed_revid})
 
-                    target.bzr_branch.unlock()
                     target.merge(source, str(proposal.reviewed_revid))
-                    target.bzr_branch.lock_write()
 
                     self.logger.debug('Firing tarmac_pre_commit hook')
                     tarmac_hooks.fire('tarmac_pre_commit',
@@ -280,12 +277,10 @@
                 if commit_message is None and self.config.imply_commit_message:
                     commit_message = proposal.description
 
-                target.bzr_branch.unlock()
                 target.commit(commit_message,
-                             revprops=revprops,
-                             authors=source.authors,
-                             reviews=self._get_reviews(proposal))
-                target.bzr_branch.lock_write()
+                              revprops=revprops,
+                              authors=source.authors,
+                              reviews=self._get_reviews(proposal))
 
                 self.logger.debug('Firing tarmac_post_commit hook')
                 tarmac_hooks.fire('tarmac_post_commit',
@@ -305,9 +300,6 @@
                               self, target)
         finally:
             target.cleanup()
-            # We're done, so unlock the branch
-            if target.bzr_branch.is_locked():
-                target.bzr_branch.unlock()
 
     def _get_mergable_proposals_for_branch(self, lp_branch):
         """Return a list of the mergable proposals for the given branch."""

=== modified file 'tarmac/branch.py'
--- tarmac/branch.py	2012-12-19 15:04:18 +0000
+++ tarmac/branch.py	2012-12-19 15:54:44 +0000
@@ -37,7 +37,6 @@
 
     def __init__(self, lp_branch, config=False, target=None):
         self.lp_branch = lp_branch
-        self.bzr_branch = bzr_branch.Branch.open(self.lp_branch.bzr_identity)
         if config:
             self.config = BranchConfig(lp_branch.bzr_identity, config)
         else:
@@ -45,15 +44,27 @@
 
         self.target = target
         self.logger = logging.getLogger('tarmac')
+        self.temp_tree_dir = None
+
+    def get_bzr_branch(self):
+        return bzr_branch.Branch.open(self.lp_branch.bzr_identity)
+
+    # For backwards compatibility
+    bzr_branch = property(get_bzr_branch)
+
+    def get_tree(self):
+        if self.temp_tree_dir is not None:
+            return WorkingTree.open(self.temp_tree_dir)
+        if os.path.exists(self.config.tree_dir):
+            return WorkingTree.open(self.config.tree_dir)
+
+    # For backwards compatibility
+    tree = property(get_tree)
 
     def __del__(self):
         """Do some potenetially necessary cleanup during deletion."""
-        try:
-            # If we were using a temp directory, then remove it
+        if self.temp_tree_dir is not None:
             shutil.rmtree(self.temp_tree_dir)
-        except AttributeError:
-            # Not using a tempdir
-            pass
 
     @classmethod
     def create(cls, lp_branch, config, create_tree=False, target=None):
@@ -62,42 +73,44 @@
             clazz.create_tree()
         return clazz
 
+
     def create_tree(self):
         '''Create the dir and working tree.'''
+        bzr_branch = self.get_bzr_branch()
         try:
-            self.logger.debug(
-                'Using tree in %(tree_dir)s' % {
-                    'tree_dir': self.config.tree_dir})
-            if os.path.exists(self.config.tree_dir):
-                self.tree = WorkingTree.open(self.config.tree_dir)
-            else:
+            tree = self.get_tree()
+            if tree is None:
                 self.logger.debug('Tree does not exist.  Creating dir')
                 # Create the path up to but not including tree_dir if it does
                 # not exist.
                 parent_dir = os.path.dirname(self.config.tree_dir)
                 if not os.path.exists(parent_dir):
                     os.makedirs(parent_dir)
-                self.tree = self.bzr_branch.create_checkout(
-                    self.config.tree_dir, lightweight=True)
+                tree = bzr_branch.create_checkout(
+                    self.config.tree_dir, lightweight=False)
+            self.logger.debug(
+                'Using tree in %(tree_dir)s' % {
+                    'tree_dir': self.config.tree_dir})
         except AttributeError:
             # Store this so we can rmtree later
             self.temp_tree_dir = tempfile.mkdtemp()
             self.logger.debug(
                 'Using temp dir at %(tree_dir)s' % {
                     'tree_dir': self.temp_tree_dir})
-            self.tree = self.bzr_branch.create_checkout(self.temp_tree_dir)
+            tree = bzr_branch.create_checkout(self.temp_tree_dir)
 
         self.cleanup()
 
     def cleanup(self):
         '''Remove the working tree from the temp dir.'''
-        assert self.tree
+        tree = self.get_tree()
+        assert tree
         self.logger.info("Running cleanup in %s." % (
             self.lp_branch.bzr_identity))
-        self.tree.revert()
+        tree.revert()
         self.logger.info("Reverted changes in %s." % (
             self.lp_branch.bzr_identity))
-        for filename in [self.tree.abspath(f) for f in self.unmanaged_files]:
+        for filename in [tree.abspath(f) for f in self.unmanaged_files]:
             if os.path.isdir(filename) and not os.path.islink(filename):
                 shutil.rmtree(filename)
             else:
@@ -105,13 +118,14 @@
 
         self.logger.info("Successfully removed extra files from %s." % (
             self.lp_branch.bzr_identity))
-        self.tree.update()
+        tree.update()
 
     def merge(self, branch, revid=None):
         '''Merge from another tarmac.branch.Branch instance.'''
-        assert self.tree
-        conflict_list = self.tree.merge_from_branch(
-            branch.bzr_branch, to_revision=revid)
+        tree = self.get_tree()
+        assert tree
+        conflict_list = tree.merge_from_branch(
+            branch.get_bzr_branch(), to_revision=revid)
         if conflict_list:
             message = u'Conflicts merging branch.'
             lp_comment = (
@@ -124,24 +138,29 @@
     @property
     def unmanaged_files(self):
         """Get the list of ignored and unknown files in the tree."""
-        self.tree.lock_read()
-        unmanaged = [x for x in self.tree.unknowns()]
-        unmanaged.extend([x[0] for x in self.tree.ignored_files()])
-        self.tree.unlock()
+        tree = self.get_tree()
+        assert tree
+        tree.lock_read()
+        unmanaged = [x for x in tree.unknowns()]
+        unmanaged.extend([x[0] for x in tree.ignored_files()])
+        tree.unlock()
         return unmanaged
 
     @property
     def conflicts(self):
         '''Print the conflicts.'''
-        assert self.tree.conflicts()
+        tree = self.get_tree()
+        assert tree
         conflicts = []
-        for conflict in self.tree.conflicts():
+        for conflict in tree.conflicts():
             conflicts.append(
                 u'%s in %s' % (conflict.typestring, conflict.path))
         return '\n'.join(conflicts)
 
     def commit(self, commit_message, revprops=None, **kwargs):
         '''Commit changes.'''
+        tree = self.get_tree()
+        assert tree
         if not revprops:
             revprops = {}
 
@@ -158,8 +177,8 @@
                                            'review identity or vote.')
             revprops['reviews'] = '\n'.join(reviews)
 
-        self.tree.commit(commit_message, committer='Tarmac',
-                         revprops=revprops, authors=authors)
+        tree.commit(commit_message, committer='Tarmac',
+                    revprops=revprops, authors=authors)
 
     @property
     def landing_candidates(self):
@@ -170,18 +189,20 @@
     def authors(self):
         author_list = []
 
+        bzr_branch = self.get_bzr_branch()
         if self.target:
-            self.bzr_branch.lock_read()
-            self.target.bzr_branch.lock_read()
+            target_bzr_branch = self.target.get_bzr_branch()
+            bzr_branch.lock_read()
+            target_bzr_branch.lock_read()
 
-            graph = self.bzr_branch.repository.get_graph(
-                self.target.bzr_branch.repository)
+            graph = bzr_branch.repository.get_graph(
+                target_bzr_branch.repository)
 
             unique_ids = graph.find_unique_ancestors(
-                self.bzr_branch.last_revision(),
-                [self.target.bzr_branch.last_revision()])
+                bzr_branch.last_revision(),
+                [target_bzr_branch.last_revision()])
 
-            revs = self.bzr_branch.repository.get_revisions(unique_ids)
+            revs = bzr_branch.repository.get_revisions(unique_ids)
             for rev in revs:
                 apparent_authors = rev.get_apparent_authors()
                 for author in apparent_authors:
@@ -189,12 +210,12 @@
                     if author not in author_list:
                         author_list.append(author)
 
-            self.target.bzr_branch.unlock()
-            self.bzr_branch.unlock()
+            target_bzr_branch.unlock()
+            bzr_branch.unlock()
         else:
-            last_rev = self.bzr_branch.last_revision()
+            last_rev = bzr_branch.last_revision()
             if last_rev != 'null:':
-                rev = self.bzr_branch.repository.get_revision(last_rev)
+                rev = bzr_branch.repository.get_revision(last_rev)
                 apparent_authors = rev.get_apparent_authors()
                 author_list.extend(
                     [a.replace('\n', '') for a in apparent_authors])
@@ -206,12 +227,13 @@
         """Return the list of bugs fixed by the branch."""
         bugs_list = []
 
-        self.bzr_branch.lock_read()
-        oldrevid = self.bzr_branch.get_rev_id(self.lp_branch.revision_count)
-        for rev_info in self.bzr_branch.iter_merge_sorted_revisions(
+        bzr_branch = self.get_bzr_branch()
+        bzr_branch.lock_read()
+        oldrevid = bzr_branch.get_rev_id(self.lp_branch.revision_count)
+        for rev_info in bzr_branch.iter_merge_sorted_revisions(
             stop_revision_id=oldrevid):
             try:
-                rev = self.bzr_branch.repository.get_revision(rev_info[0])
+                rev = bzr_branch.repository.get_revision(rev_info[0])
                 for bug in rev.iter_bugs():
                     if bug[0].startswith('https://launchpad.net/bugs/'):
                         bugs_list.append(bug[0].replace(
@@ -219,5 +241,5 @@
             except NoSuchRevision:
                 continue
 
-        self.bzr_branch.unlock()
+        bzr_branch.unlock()
         return bugs_list

=== modified file 'tarmac/tests/test_branch.py'
--- tarmac/tests/test_branch.py	2012-06-29 20:03:55 +0000
+++ tarmac/tests/test_branch.py	2012-12-19 15:54:44 +0000
@@ -38,7 +38,6 @@
         a_branch = branch.Branch.create(MockLPBranch(tree_dir), self.config)
         self.assertTrue(isinstance(a_branch, branch.Branch))
         self.assertTrue(a_branch.lp_branch.bzr_identity is not None)
-        self.assertFalse(hasattr(a_branch, 'tree'))
         self.remove_branch_config(tree_dir)
 
     def test_create_missing_parent_dir(self):
@@ -66,19 +65,6 @@
         self.assertTrue(self.branch1.lp_branch.bzr_identity is not None)
         self.assertTrue(hasattr(self.branch1, 'tree'))
 
-    def test_merge_raises_exception_with_no_tree(self):
-        '''A merge on a branch with no tree will raise an exception.'''
-        branch3_dir = os.path.join(self.TEST_ROOT, 'branch3')
-        self.add_branch_config(branch3_dir)
-        branch3 = branch.Branch.create(MockLPBranch(
-                branch3_dir, source_branch=self.branch1.lp_branch),
-                                       self.config)
-
-        self.assertRaises(
-            AttributeError, branch3.merge, self.branch2)
-        self.remove_branch_config(branch3_dir)
-        shutil.rmtree(branch3_dir)
-
     def test_merge_no_changes(self):
         '''A merge on a branch with a tree will raise an exception if no
         changes are present.'''

